# frozen_string_literal: true

require 'spec_helper'

module LetItCrash
  module Builders
    RSpec.describe FileBuilder do
      describe '.build' do
        let(:filename) { 'spec/fixtures/source.file' }
        let(:lines) do
          [
            instance_double(
              SimpleCov::SourceFile::Line,
              line_number: 1,
              skipped?: false,
              coverage: 0,
            ),
            instance_double(
              SimpleCov::SourceFile::Line,
              line_number: 2,
              skipped?: false,
              coverage: 0,
            ),
            instance_double(
              SimpleCov::SourceFile::Line,
              line_number: 3,
              skipped?: false,
              coverage: 1,
            )
          ]
        end
        let(:file) do
          instance_double(
            SimpleCov::SourceFile,
            filename: filename,
            lines: lines,
          )
        end
        let(:result) { described_class.build(file: file, rewriter: rewriter) }

        context 'without rewriter' do
          let(:rewriter) { nil }

          it { expect(result).to be_a Proto::File }
          it { expect(result.coverage).to eq 1.0 }
          it { expect(result.digest).to start_with '73054' }

          context 'with lines' do
            let(:result_lines) { result.lines }

            it { expect(result_lines.count).to eq 1 }
            it { expect(result_lines[3]).to eq 1 }
          end # context 'with lines'

          context 'with path' do
            let(:path) { result.path }

            it { expect(path.relative).to eq filename }
            it { expect(path.rewritten).to be_empty }
          end # context 'with path'
        end # context 'without rewriter'

        context 'with rewriter' do
          let(:rewriter) { ->(name) { "bacon/#{name}" } }

          it { expect(result.path.rewritten).to start_with 'bacon/spec' }
        end # context 'with rewriter'

        context 'with no relevant lines' do
          let(:rewriter) { nil }
          let(:lines)    { [] }

          it { expect(result.coverage).to eq 1 }
        end # context 'with no relevant lines'
      end # describe '.build'
    end # RSpec.describe FileBuilder
  end # module Builders
end # module LetItCrash
