# frozen_string_literal: true

require 'spec_helper'

# rubocop:disable RSpec/MessageSpies
module LetItCrash
  module Builders
    describe ReportBuilder do
      describe '.build' do
        let(:branch)      { 'branch' }
        let(:sha)         { 'sha' }
        let(:environment) { nil }
        let(:rewriter)    { nil }
        let(:report) do
          described_class.build(
            environment: environment,
            result: result,
            rewriter: rewriter,
          )
        end

        before do
          expect(Matcher)
            .to receive(:new)
            .with(environment: environment)
            .and_return(
              instance_double(
                Matcher,
                matches?: matches,
                branch: branch,
                sha: sha,
              ),
            )
        end

        context 'with match' do
          let(:matches) { true }
          let(:file)    { instance_double(SimpleCov::SourceFile) }
          let(:result)  { instance_double(SimpleCov::Result, files: [file]) }

          before do
            expect(FileBuilder)
              .to receive(:build)
              .with(file: file, rewriter: rewriter)
              .and_return(Proto::File.new)
          end

          it { expect(report.commit.branch).to eq branch }
          it { expect(report.commit.sha).to eq sha }
          it { expect(report.files.count).to eq 1 }
        end # context 'with match'

        context 'with no match' do
          let(:matches) { false }
          let(:result)  { nil }

          it { expect { report }.to raise_error ReportBuilder::MatcherMissing }
        end # context 'with no match'
      end # describe '.build'
    end # describe ReportBuilder
  end # module Builders
end # module LetItCrash
