# frozen_string_literal: true

require 'spec_helper'

# rubocop:disable RSpec/MessageSpies
module LetItCrash
  RSpec.describe Formatter do
    let(:mock_reporter) { instance_double(Reporters::Upload) }

    describe '#initialize' do
      let(:default_call) do
        expect(Reporters::Upload)
          .to receive(:from_env)
          .and_return(mock_reporter)
      end

      context 'without reporters' do
        let(:formatter) { described_class.new }

        before { default_call.once }

        it { expect(formatter).to be_a described_class }
      end # context 'without reporters'

      context 'with reporters' do
        let(:formatter) { described_class.new(reporters: [mock_reporter]) }

        before { default_call.never }

        it { expect(formatter).to be_a described_class }
      end # context 'with reporters'
    end # describe '#initialize'

    describe '#format' do
      let(:formatter)   { described_class.new(reporters: [mock_reporter]) }
      let(:mock_result) { instance_double(SimpleCov::Result) }
      let(:mock_report) { instance_double(Proto::Report) }

      before do
        expect(Builders::ReportBuilder)
          .to receive(:build)
          .with(environment: ENV, result: mock_result, rewriter: nil)
          .and_return(mock_report)

        expect(mock_reporter).to receive(:report).with(mock_report)
      end

      it { expect { formatter.format(mock_result) }.not_to raise_error }
    end # describe '#format'

    describe '#new' do
      let(:formatter) { described_class.new(reporters: [mock_reporter]) }

      it { expect(formatter.new).to eq formatter }
    end # describe '#new'
  end # RSpec.describe Formatter
end # module LetItCrash
