# frozen_string_literal: true

require 'spec_helper'

# rubocop:disable RSpec/MessageSpies
module LetItCrash
  module Matchers
    RSpec.describe Git do
      let(:matcher) { described_class.new(environment: nil) }

      shared_context 'with_command' do |cmd, result|
        let(:output) { instance_double(IO, read: result) }

        before { expect(IO).to receive(:popen).with(cmd).and_return(output) }
      end # shared_context 'with_command'

      describe '#matches?' do
        let(:result) { matcher.matches? }

        context 'when on a Git branch' do
          include_context 'with_command',
                          described_class::STATUS_CMD,
                          'On branch master'

          it { expect(result).to be_truthy }
        end # context 'when on a Git branch'

        context 'when not on a Git branch' do
          include_context 'with_command', 'git status', 'fatal: Not a git...'

          it { expect(result).to be_falsey }
        end # context 'when not on a Git branch'
      end # describe '#matches?'

      describe '#branch' do
        context 'when HEAD' do
          include_context 'with_command',
                          described_class::BRANCH_CMD,
                          'HEAD'

          it { expect(matcher.branch).to eq 'master' }
        end # context 'when HEAD'

        context 'with normal branch' do
          include_context 'with_command',
                          described_class::BRANCH_CMD,
                          'branch'

          it { expect(matcher.branch).to eq 'branch' }
        end # context 'with normal branch'
      end # describe '#branch'

      describe '#sha' do
        include_context 'with_command', described_class::SHA_CMD, 'sha'

        it { expect(matcher.sha).to eq 'sha' }
      end # describe '#sha'
    end # RSpec.describe Git
  end # module Matchers
end # module LetItCrash
