# frozen_string_literal: true

require 'spec_helper'
require 'matchers/shared_helpers'

module LetItCrash
  module Matchers
    RSpec.describe Shippable do
      include_context 'with_branch_and_sha'
      let(:ci)        { 'true' }
      let(:shippable) { 'true' }
      let(:environment) do
        {
          'CI' => ci,
          'SHIPPABLE' => shippable,
          'BRANCH' => branch,
          'COMMIT' => sha
        }
      end

      it_behaves_like 'reports_branch_and_sha'

      describe '#matches?' do
        let(:result) { matcher.matches? }

        context 'with CI and SHIPPABLE set to `true` (default)' do
          it { expect(result).to be_truthy }
        end # context 'with CI and SHIPPABLE set to `true` (default)'

        context 'with CI set to nil' do
          let(:ci) { nil }

          it { expect(result).to be_falsey }
        end # context 'with CI set to nil'

        context 'with SHIPPABLE set to nil' do
          let(:shippable) { nil }

          it { expect(result).to be_falsey }
        end # context 'with SHIPPABLE set to nil'
      end # describe '#matches?'
    end # RSpec.describe Shippable
  end # module Matchers
end # module LetItCrash
