# frozen_string_literal: true

require 'spec_helper'

# rubocop:disable RSpec/VerifiedDoubles, RSpec/MessageSpies
module LetItCrash
  module Reporters
    RSpec.describe Upload do
      let(:message)  { 'message' }
      let(:input)    { double(to_json: message) }
      let(:endpoint) { 'endpoint' }
      let(:token)    { 'token' }

      describe '.from_env' do
        let(:reporter) { described_class.from_env }

        before do
          expect(ENV)
            .to receive(:fetch)
            .with('LETITCRASH_COVERAGE_ENDPOINT', start_with('https://'))
            .and_return(endpoint)
          expect(ENV)
            .to receive(:fetch)
            .with('LETITCRASH_COVERAGE_TOKEN')
            .and_return(token)
        end

        it { expect(reporter).to be_a described_class }
      end # describe '.from_env'

      describe '#report' do
        let(:reporter) { described_class.new(endpoint: endpoint, token: token) }
        let(:result) { reporter.report(input) }

        before do
          expect(RestClient)
            .to receive(:post)
            .with(endpoint, message, include('X-LetItCrash-Token' => token))
        end

        it { expect { result }.not_to raise_error }
      end # describe '#report'
    end # RSpec.describe Upload
  end # module Reporters
end # module LetItCrash
